// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest;

/// Typedef for _OmnitestFabric._processRequest
typedef List<OmnibusResponse> _ProcessRequest(
    OmnitestRequest request, List<OmnitestResponse> moduleResponses);

/// Omnitest testing system
class _OmnitestFabric {
  /// General bus
  Omnibus bus;

  /// Event logger
  OmnitestEventLogger eventLogger = new OmnitestEventLogger();

  /// Request handler
  OmnitestGlobalRequestHandler requestHandler;

  /// Request processing callback
  _ProcessRequest _processRequest;

  /// Constructor
  _OmnitestFabric(this._processRequest) {
    requestHandler = new OmnitestGlobalRequestHandler(_globalRequestHandler);
    bus = new Omnibus();
    bus.bind(eventLogger);
    bus.bind(requestHandler);
  }

  /// Global request handler
  Future<List<OmnibusResponse>> _globalRequestHandler(
      OmnibusRequest request) async {
    // Check what responses are provided by the module itself.
    List<OmnibusResponse> moduleResponses =
        await (await bus.publishRequest(request)).onComplete();

    // Convert all responses to OmnitestResponse to make them comparable.
    var omnitestResponses = new List<OmnitestResponse>();
    moduleResponses.forEach((OmnibusResponse response) => omnitestResponses
        .add(new OmnitestResponse(response.omnibusId, response.toPds())));

    // Return mocked responses.
    return _processRequest(
        new OmnitestRequest(request.omnibusId, request.toPds()),
        omnitestResponses);
  }
}
